import typing as t
import flask

from kvcommon.logger import get_logger

LOG = get_logger("kvc-flask")

FLASK_CONTEXT_PREFIX = "kvc-flask_"


def set_context_prefix(prefix: str):
    global FLASK_CONTEXT_PREFIX
    FLASK_CONTEXT_PREFIX = prefix or FLASK_CONTEXT_PREFIX


def get_flask_context_local(
    key: str, default: t.Optional[t.Any] = None, prefix: str = FLASK_CONTEXT_PREFIX
):
    key = f"{prefix}{key}"
    return flask.g.get(key, default)


def set_flask_context_local(key: str, value: t.Any = None, prefix: str = FLASK_CONTEXT_PREFIX):
    key = f"{prefix}{key}"
    return setattr(flask.g, key, value)


def get_config_value(config_key: str):
    return flask.current_app.config.get(config_key)


def get_config_value_bool(config_key: str) -> bool:
    val = get_config_value(config_key)
    if isinstance(val, str):
        if val.lower() in ["true", "yes", "y"]:
            val = True
        elif not val or val.lower() in ["false", "no", "n"]:
            val = False
    if not isinstance(val, bool):
        raise TypeError(
            f"Retrieved non-bool value for get_config_value_bool - key={config_key}, value={val}"
        )
    return val



from prometheus_client import start_http_server

from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED as ENABLED
from kvcommon_flask.vars import KVC_FLASK_METRICS_PORT
from .metrics import SCHEDULER_JOB_EVENT

from .metrics import APP_INFO
from .metrics import SERVER_REQUEST_SECONDS
from .metrics import incr
from .metrics import decr


def init_metrics():
    if ENABLED:
        start_http_server(KVC_FLASK_METRICS_PORT)

__all__ = [
    "ENABLED",
    "incr",
    "decr",
    "init_metrics",
    "SCHEDULER_JOB_EVENT",
    "APP_INFO",
    "SERVER_REQUEST_SECONDS",
]



import os
import typing as t

from kvcommon.types import to_bool

KVC_FLASK_METRICS_PORT = int(os.getenv("KVC_FLASK_METRICS_PORT", 9090))
KVC_FLASK_METRICS_ENABLED = to_bool(os.getenv("KVC_FLASK_METRICS_ENABLED", False))
KVC_FLASK_TRACES_ENABLED = to_bool(os.getenv("KVC_FLASK_TRACES_ENABLED", False))



from prometheus_client import Counter
from prometheus_client import Gauge
from prometheus_client import Histogram
from prometheus_client import Info
from prometheus_client import Summary

# https://prometheus.io/docs/practices/naming/


def incr(metric: Counter | Gauge):
    from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED

    if KVC_FLASK_METRICS_ENABLED:
        metric.inc()

def decr(gauge: Gauge):
    from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED

    if KVC_FLASK_METRICS_ENABLED:
        gauge.dec()


def set_app_info(app_version: str):
    from kvcommon_flask.vars import KVC_FLASK_METRICS_ENABLED

    if KVC_FLASK_METRICS_ENABLED:
        APP_INFO.info(dict(version=app_version))


APP_INFO = Info("app", "Application info")


SCHEDULER_JOB_EVENT = Counter(
    "scheduler_job_event_total",
    "Counter of scheduled job events by event enum",
    labelnames=[
        "job_id",
        "event",
    ],
)

# Total time spent from start to finish on a request
SERVER_REQUEST_SECONDS = Histogram(
    "server_request_seconds",
    "Time taken for server to handle request",
    labelnames=["path"],
)
